
BEGIN {
    $CameraStore::SET_DB = ['Main',
               "dbi:mysql:testidb",
               'root', ''];
}

use warnings;
use strict;
use Test::More qw(no_plan);
use CameraStore;
use Log::Log4perl qw(:easy);
use Getopt::Std;
use File::Path;

my $IMG_ID = "2004-01-19_11:29:13";

    # Figure out where test images are
my $TEST_DIR = "t";
$TEST_DIR = "." unless -d $TEST_DIR;
die "Cannot find image dir" unless -d $TEST_DIR;

my $IMG_FILE_DIR = "$TEST_DIR/store";
mkdir $IMG_FILE_DIR unless -d $IMG_FILE_DIR;

Log::Log4perl->easy_init(
    { file   => 'stderr',
      level  => $FATAL,
      layout => "%F-%M: %m%n"});

#############################################################
# Capture STDOUT to a temporary file and a filehandle to read from it
#############################################################
open STDOUT, ">$TEST_DIR/log";
select STDOUT; $| = 1; #needed on win32
open IN, "<$TEST_DIR/log" or die "Cannot open $TEST_DIR/log";
sub readstdout { return join("", <IN>); }

#############################################################
# Insert test image to DB
#############################################################
my %OPTS = ("i" => 1);
@ARGV = ("$TEST_DIR/img/test.jpg");

my $rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);

ok($rc, "Insert Image");

#############################################################
# List its tags (non-existent)
#############################################################
%OPTS = ("l" => 1);
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);

ok($rc, "List tags");
is(readstdout(), "$TEST_DIR/img/test.jpg: \n", "List tags output");

#############################################################
# Add two tags
#############################################################
for(1..2) {
    %OPTS = ("t" => "tag$_");
    @ARGV = ("$TEST_DIR/img/test.jpg");

    $rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                  cmd_line_options => \%OPTS);

    ok($rc, "Adding two tags");
}

#############################################################
# List its tags (Two should be there now)
#############################################################
%OPTS = ("l" => 1);
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);
ok($rc, "List tags");
like(readstdout(), qr#$TEST_DIR/img/test.jpg: tag(1|2), tag(1|2)\n#, 
   "List tags output");

#############################################################
# Remove two tags
#############################################################
for my $tag (1..2) {
    %OPTS = ("u" => "tag$tag");
    @ARGV = ("$TEST_DIR/img/test.jpg");

    $rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                  cmd_line_options => \%OPTS);

    ok($rc, "Removing tag tag$tag");
}

#############################################################
# List its tags (non-existent)
#############################################################
%OPTS = ("l" => 1);
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);

ok($rc, "List tags");
is(readstdout(), "$TEST_DIR/img/test.jpg: \n", "List tags output");

#############################################################
# Add two tags at once
#############################################################
%OPTS = ("t" => "tag1|tag2");
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                              cmd_line_options => \%OPTS);

ok($rc, "Adding two tags");

#############################################################
# List its tags (Two should be there now)
#############################################################
%OPTS = ("l" => 1);
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);
ok($rc, "List tags");
like(readstdout(), qr#$TEST_DIR/img/test.jpg: tag(1|2), tag(1|2)\n#, 
   "List tags output");

#############################################################
# Insert test image again to DB
#############################################################
%OPTS = ("i" => 1);
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);

ok(! defined $rc, "Insert Dupe Image");

#############################################################
# Insert another instance of the test image into DB
#############################################################
%OPTS = ("i" => 1, "d" => 1, "t" => 'tag3|tag4');
@ARGV = ("$TEST_DIR/img/test.jpg");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                                 cmd_line_options => \%OPTS);

ok(defined $rc, "Insert 2nd image with same timestamp");

#############################################################
# Search both instances by tag
#############################################################
%OPTS = ("s" => '%tag%');
@ARGV = ();

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                              cmd_line_options => \%OPTS);
ok($rc, "Search with -s");
like(readstdout(), qr#$IMG_ID.*?$IMG_ID#s, "Check for both images");

#############################################################
# Pipe both ids into a -l
#############################################################
%OPTS = ("l" => '1');
@ARGV = ($IMG_ID, "${IMG_ID}_1");

$rc = CameraStore::Shell->run(img_file_dir => $IMG_FILE_DIR, 
                              cmd_line_options => \%OPTS);
ok($rc, "List tags");
my $output = readstdout();
like($output, qr#$IMG_ID: tag(1|2), tag(1|2)\n#, "List tags output");
like($output, qr#${IMG_ID}_1: tag(3|4), tag(3|4)\n#, "List tags output");

#############################################################
# Delete image and tags from DB
#############################################################

my $db = CameraStore->new();
ok($db->delete_image($IMG_ID), "Deleting image");
ok($db->delete_image("${IMG_ID}_1"), "Deleting image");

#############################################################
# Clean up
#############################################################
ok(rmtree($IMG_FILE_DIR), "rmtree");
ok(unlink("$TEST_DIR/log"), "unlink log file");
